@@ -74,7 +74,7 @@ Caveats
   executed.  For this reason coverage analysis now uses trace.py which is
   slower, but more accurate.
 
-Copyright (c) 2004--2012 Marius Gedminas <marius@pov.lt>
+Copyright (c) 2004--2014 Marius Gedminas <marius@pov.lt>
 Copyright (c) 2007 Hanno Schlichting
 Copyright (c) 2008 Florian Schulze
 
@@ -101,11 +101,11 @@ Released under the MIT licence since December 2006:
 (Previously it was distributed under the GNU General Public Licence.)
 """
 
-__author__ = "Marius Gedminas (marius@gedmin.as)"
-__copyright__ = "Copyright 2004-2013 Marius Gedminas"
+__author__ = "Marius Gedminas <marius@gedmin.as>"
+__copyright__ = "Copyright 2004-2014 Marius Gedminas"
 __license__ = "MIT"
-__version__ = "1.7"
-__date__ = "2013-10-16"
+__version__ = "1.7.1"
+__date__ = "2014-12-02"
 
 
 import atexit
@@ -221,12 +221,10 @@ def profile(fn=None, skip=0, filename=None, immediate=False, dirs=False,
             break
     else:
         raise ValueError('only these profilers are available: %s'
-                             % ', '.join(AVAILABLE_PROFILERS))
+                             % ', '.join(sorted(AVAILABLE_PROFILERS)))
     fp = profiler_class(fn, skip=skip, filename=filename,
                         immediate=immediate, dirs=dirs,
                         sort=sort, entries=entries)
-    # fp = HotShotFuncProfile(fn, skip=skip, filename=filename, ...)
-         # or HotShotFuncProfile
     # We cannot return fp or fp.__call__ directly as that would break method
     # definitions, instead we need to return a plain function.
 
@@ -355,7 +353,7 @@ class FuncProfile(object):
         print("*** PROFILER RESULTS ***")
         print("%s (%s:%s)" % (funcname, filename, lineno))
         if self.skipped:
-            skipped = "(%d calls not profiled)" % self.skipped
+            skipped = " (%d calls not profiled)" % self.skipped
         else:
             skipped = ""
         print("function called %d times%s" % (self.ncalls, skipped))
@@ -399,13 +397,14 @@ if cProfile is not None:
 
 if hotshot is not None:
 
-    class HotShotFuncProfile(object):
+    class HotShotFuncProfile(FuncProfile):
         """Profiler for a function (uses hotshot)."""
 
         # This flag is shared between all instances
         in_profiler = False
 
-        def __init__(self, fn, skip=0, filename=None):
+        def __init__(self, fn, skip=0, filename=None, immediate=False,
+                     dirs=False, sort=None, entries=40):
             """Creates a profiler for a function.
 
             Every profiler has its own log file (the name of which is derived
@@ -417,17 +416,13 @@ if hotshot is not None:
             The log file is not removed and remains there to clutter the
             current working directory.
             """
-            self.fn = fn
-            self.filename = filename
-            if self.filename:
+            if filename:
                 self.logfilename = filename + ".raw"
             else:
                 self.logfilename = fn.__name__ + ".prof"
-            self.profiler = hotshot.Profile(self.logfilename)
-            self.ncalls = 0
-            self.skip = skip
-            self.skipped = 0
-            atexit.register(self.atexit)
+            super(HotShotFuncProfile, self).__init__(
+                fn, skip=skip, filename=filename, immediate=immediate,
+                dirs=dirs, sort=sort, entries=entries)
 
         def __call__(self, *args, **kw):
             """Profile a singe call to the function."""
@@ -439,39 +434,29 @@ if hotshot is not None:
             if HotShotFuncProfile.in_profiler:
                 # handle recursive calls
                 return self.fn(*args, **kw)
+            if self.profiler is None:
+                self.profiler = hotshot.Profile(self.logfilename)
             try:
                 HotShotFuncProfile.in_profiler = True
                 return self.profiler.runcall(self.fn, *args, **kw)
             finally:
                 HotShotFuncProfile.in_profiler = False
+                if self.immediate:
+                    self.print_stats()
+                    self.reset_stats()
 
-        def atexit(self):
-            """Stop profiling and print profile information to sys.stderr.
-
-            This function is registered as an atexit hook.
-            """
-            self.profiler.close()
-            funcname = self.fn.__name__
-            filename = self.fn.__code__.co_filename
-            lineno = self.fn.__code__.co_firstlineno
-            print("")
-            print("*** PROFILER RESULTS ***")
-            print("%s (%s:%s)" % (funcname, filename, lineno))
-            if self.skipped:
-                skipped = "(%d calls not profiled)" % self.skipped
+        def print_stats(self):
+            if self.profiler is None:
+                self.stats = pstats.Stats(Profile())
             else:
-                skipped = ""
-            print("function called %d times%s" % (self.ncalls, skipped))
-            print("")
-            stats = hotshot.stats.load(self.logfilename)
-            # hotshot.stats.load takes ages, and the .prof file eats megabytes, but
-            # a saved stats object is small and fast
-            if self.filename:
-                stats.dump_stats(self.filename)
-            # it is best to save before strip_dirs
-            stats.strip_dirs()
-            stats.sort_stats('cumulative', 'time', 'calls')
-            stats.print_stats(40)
+                self.profiler.close()
+                self.stats = hotshot.stats.load(self.logfilename)
+            super(HotShotFuncProfile, self).print_stats()
+
+        def reset_stats(self):
+            self.profiler = None
+            self.ncalls = 0
+            self.skipped = 0
 
     AVAILABLE_PROFILERS['hotshot'] = HotShotFuncProfile
 
@@ -505,7 +490,11 @@ if hotshot is not None:
         def __call__(self, *args, **kw):
             """Profile a singe call to the function."""
             self.ncalls += 1
-            return self.profiler.runcall(self.fn, args, kw)
+            old_trace = sys.gettrace()
+            try:
+                return self.profiler.runcall(self.fn, args, kw)
+            finally: # pragma: nocover
+                sys.settrace(old_trace)
 
         def atexit(self):
             """Stop profiling and print profile information to sys.stderr.
@@ -538,6 +527,9 @@ if hotshot is not None:
                     fs.mark(lineno)
             reader.close()
             print(fs)
+            never_executed = fs.count_never_executed()
+            if never_executed:
+                print("%d lines were not executed." % never_executed)
 
 
 class TraceFuncCoverage:
@@ -574,12 +566,14 @@ class TraceFuncCoverage:
     def __call__(self, *args, **kw):
         """Profile a singe call to the function."""
         self.ncalls += 1
-        if TraceFuncCoverage.tracing:
+        if TraceFuncCoverage.tracing: # pragma: nocover
             return self.fn(*args, **kw)
+        old_trace = sys.gettrace()
         try:
             TraceFuncCoverage.tracing = True
             return self.tracer.runfunc(self.fn, *args, **kw)
-        finally:
+        finally: # pragma: nocover
+            sys.settrace(old_trace)
             TraceFuncCoverage.tracing = False
 
     def atexit(self):
@@ -634,7 +628,8 @@ class FuncSource:
             self.sourcelines.setdefault(lineno, 0)
         if lines:
             self.firstcodelineno = min(lines)
-        else:
+        else:  # pragma: nocover
+            # This branch cannot be reached, I'm just being paranoid.
             self.firstcodelineno = self.firstlineno
 
     def mark(self, lineno, count=1):
@@ -666,7 +661,10 @@ class FuncSource:
             if counter is None:
                 prefix = ' ' * 7
             elif counter == 0:
-                if self.blank_rx.match(line):
+                if self.blank_rx.match(line):  # pragma: nocover
+                    # This is an workaround for an ancient bug I can't
+                    # reproduce, perhaps because it was fixed, or perhaps
+                    # because I can't remember all the details.
                     prefix = ' ' * 7
                 else:
                     prefix = '>' * 6 + ' '
@@ -677,7 +675,7 @@ class FuncSource:
         return ''.join(lines)
 
 
-def timecall(fn=None, immediate=True, timer=time.time):
+def timecall(fn=None, immediate=True, timer=None):
     """Wrap `fn` and print its execution time.
 
     Example::
@@ -704,6 +702,8 @@ def timecall(fn=None, immediate=True, timer=time.time):
             return timecall(fn, immediate=immediate, timer=timer)
         return decorator
     # @timecall syntax -- we are a decorator.
+    if timer is None:
+        timer = time.time
     fp = FuncTimer(fn, immediate=immediate, timer=timer)
     # We cannot return fp or fp.__call__ directly as that would break method
     # definitions, instead we need to return a plain function.
